import { Component, writeNotify } from "pi_ucore/util/ecs";
import { BombType } from "../single/bomb_list";

export class TypeC extends Component {
    value: DataType; // 物品类型
    subType: NormalLockType | SpecialLockType | BombType | AwardType | EventDataType; // 子类型
    subTypeArg: string; // 子类型参数 例如袋子ID
    hideType: BombType | AwardType; // 隐藏类型 炸开后切换
    orgNeedBombTimes: number = 1; // 原本需要被炸次数
    cannotBomb: boolean = false; // 不能炸
    _needBombTimes: number = 1; // 需要被炸次数 有些物品需要被炸多次

    @writeNotify
    set needBombTimes(val: number) {
        // console.log('修改needBombTimes', this.__id__, this._needBombTimes)
        this._needBombTimes = val;
    }
    get needBombTimes(): number {
        return this._needBombTimes;
    }

    constructor(type: DataType, needBombTimes: number = 1, cannotBomb: boolean = false, subType?: any, subTypeArg?: string, hideType?: any) {
        super();
        this.value = type;
        this.orgNeedBombTimes = this._needBombTimes = needBombTimes;
        this.cannotBomb = cannotBomb;
        this.subType = subType;
        this.subTypeArg = subTypeArg;
        this.hideType = hideType;
    }
    // 空位置
    public isEmpty() {
        return this.subType === SpecialLockType.OpenSpace || this.subType === SpecialLockType.OpenSpaceObstacle
    }
    // 不能放炸弹的位置
    public isEmptyNoBoom() {
        const subTypeList = [
            SpecialLockType.OpenSpace, SpecialLockType.OpenSpaceObstacle, SpecialLockType.BombDevice
            , NormalLockType.Foam, NormalLockType.PearlTray
        ]
        return subTypeList.includes(<NormalLockType | SpecialLockType>this.subType)
    }
}

export class LockTypeC extends Component {
    blockBombExpansion: boolean = false; // 阻挡横竖雷扩展
    aroundBombEffect: boolean = false; // 周围爆炸是否影响自身
    cannotHoldData: boolean = false; // 不能压住物品
    recordMultipleTimes: boolean = false; // 周围爆炸记录多次

    constructor(blockBombExpansion: boolean = false, aroundBombEffect: boolean = false, cannotHoldData: boolean = false, recordMultipleTimes: boolean = false) {
        super();
        this.blockBombExpansion = blockBombExpansion;
        this.aroundBombEffect = aroundBombEffect;
        this.cannotHoldData = cannotHoldData;
        this.recordMultipleTimes = recordMultipleTimes;
    }
}

// 物品类型
export enum DataType {
    /**
     * 基础物品
     */
    Apple = 1, // 苹果
    Pear = 2, // 梨
    Grape = 3, // 葡萄
    Banana = 4, // 香蕉
    Orange = 5, // 橘子
    Mango = 6, // 芒果
    Litchi = 7, // 荔枝
    EmptyBox = 8, // 空箱子 不藏奖励
    /**
     * 特殊物品
     */
    Cherry = 9, // 樱桃 不能炸 暴露就收集
    Donut = 10, // 甜甜圈(雕塑) 不能炸 到底部收集
    Medusa = 11, // 美杜莎 炸多次 随机石化其他格子

    /**
     * 锁 大类型
     */
    NormalLock = 100, // 普通锁
    SpecialLock = 150, // 特殊锁 不能炸

    /**
     * 其他大类型
     */
    SynthesisBomb = 200, // 合成炸弹
    AwardData = 300, // 奖励物品 炸普通物品后可能获得
    EventData = 400, // 事件物品 合成炸弹时可算作任意普通物品
    Haystack = 600, // 草垛(放在草坪层) 草垛后面有隐藏物品 其他物品可掉落在其上面，炸物品时也炸草垛，如果其上的物品不能消失则草垛也不能消失
    SynthesisData = 700, // 合成物品
    TimeAwardData = 800, // 定时奖励物品 创建回合不允许被炸

    /**
     * 目标物品 不创建格子 仅用来收集
     */
    FishScale = 501, // 鱼鳞
    Pearl = 502, // 珍珠
    Lawn = 503, //  草坪（水域）
    MermaidLock = 504, //美人鱼锁链
    Mail = 505, // 信封
}


// 普通锁
export enum NormalLockType {
    Chain = 101, // 锁链
    Jelly = 102, // 果冻 可多层 炸周围也消除一层
    Container = 103, // 集装箱 不压住物品 可多层 炸周围也消除一层
    Biscuit = 104, // 饼干 不压住物品 可多层 炸周围也消除一层 阻挡横竖雷
    Petrifaction = 105, // 石化
    Clownfish = 106, // 小丑鱼 炸自身可产出一个鱼鳞
    Blinds = 107, // 百叶窗 不压住物品 炸周围或自身向下卷起一格 阻挡草坪扩展
    Foam = 108, // 泡沫 炸周围也消除 行动未消除泡沫则把相邻一格变为泡沫
    PearlTray = 109, // 珍珠盘 炸周围或自身获得一个珍珠
    Mermaid = 110, // 美人鱼 炸周围或自身解开一个美人鱼禁制(固定8个禁制, 固定占四格)
    Ice = 111, // 冰块，与锁链相同的模式，会出现多张图的切换 炸周围也消除
    MailBox = 112, // 信箱 不压住物品 炸周围或自身产出一个信封
}

// 特殊锁 不能炸
export enum SpecialLockType {
    OpenSpace = 151, // 空地
    Obstacle = 152, // 障碍物
    Curtain = 153, // 窗帘 收集固定数量的对应颜色
    BombDevice = 154, // 炸弹装置 随机产出不同的雷
    OpenSpaceObstacle = 155, // 空地障碍物   已障碍物的形式表达的空地
}

// 奖品类型
export enum AwardType {
    GlodBag = 301, // 金袋子
    BombBag = 302, // 炸弹袋子 一格炸弹
    NormalBag = 303, // 普通袋子
}

// 事件物品类型
export enum EventDataType {
    Sword = 401, // 剑
    Shield = 402, // 盾
    RedMonster = 403, // 红色怪物 炸两次
    YellowMonster = 404, // 黄色怪物
    GlodBag = 405, // 金袋子 炸两次

    Hero = 415, // 英雄
}