import { writeNotify } from "pi_ucore/util/ecs";
import { BombType } from "./bomb_list";
import { DataType } from "../component/type";

// 动画数组
export class AnimList {
    animList: Map<number, AnimData[]> = new Map; // 动画分组
    overTimeList: Map<number, number> = new Map; // 结束时间
    @writeNotify
    cur: AnimData; // 当前动画
    overTime: number = 0; // 总结束时间

    // 塞入动画
    addAnimData(id: number, animData: AnimData) {
        let arr = this.animList.get(id) || []
        arr.push(animData);
        this.animList.set(id, arr);
    }
}

export type AnimData = {
    type: AnimType; // 动画类型
    time: number; // 动画时间
    arg?: BombType; // 炸弹类型
    posList: number[][]; // 影响的所有坐标
    startPos?: number[][]; // 飞行动画起点坐标
    dataType?: DataType; // 合成动画物品类型
    rocketEndPos?: number[][]; // 火箭终点坐标
    otherEndPos?: number[][][]; // 其他终点坐标
    entityKey?: number; // entityKey
    isAnnex?: boolean;// 处理的是合成
}
export enum AnimType {
    Exploding = 1, // 爆炸
    AirWave = 2, // 气浪
    BombWire = 3, // 炸弹导线
    SynthesisBomb = 4, // 合成炸弹
    Flying = 5, // 飞行
    CloseCurtains = 6, // 关窗帘
    LayLawn = 7, // 铺草坪
    FishJump = 8, //跳鱼(甜甜圈)
    UnlockMermaid = 9, //解锁美人鱼
    MedusaAttack = 10, //美杜莎攻击
    BlindsCurlUp = 11, //百叶窗卷起
    AutoSynthesisBomb = 12, //自动合成动画
    AutoAddBomb = 13,// 场地自动加炸弹
    Shuffle = 14,// 节点混乱
    Swap = 15,// 节点交换
    BombDevice = 16,// 炸弹装置
    SynthesisData = 17,// 合成物品
    ExplodingAni = 18, // 爆炸动画

}

export const AnimTime = {
    Exploding: 100, // 爆炸
    ExplodingAni: 500, // 爆炸动画
    AirWave: 300, // 气浪
    BombWire: 100, // 炸弹导线
    SynthesisBomb: 500, // 合成炸弹
    AutoSynthesisBomb: 2200, //自动合成动画
    Flying: 1000, // 飞行
    CloseCurtains: 200, // 关窗帘
    LayLawn: 100, // 铺草坪
    FishJump: 1000, //跳鱼
    UnlockMermaid: 1000, //解锁美人鱼
    MedusaAttack: 800, //美杜莎攻击
    BlindsCurlUp: 800, //百叶窗卷起
    AutoAddBomb: 300,// 场地自动加炸弹
    Shuffle: 300,// 节点混乱
    Swap: 100,// 节点混乱
    BombDevice: 1200,// 炸弹装置
    SynthesisData: 500,// 合成物品
}