import { Entity, Multi, Single, System, World, read, ArrMulti, write, listenModify, ModifyEvent } from "pi_ucore/util/ecs";
import { DataC, DataState, LayerType, StateC } from "../component/data";
import { GlobalState } from "../single/global_state";
import { FrameTime } from "pi_scene/scene/single/global";
import { ActionCmd, ActionName } from "../single/action_cmd";
import { SquarePosFour } from "../utils/base";
import { TypeC } from "../component/type";

/**
 * 自动消除
 */
export class MatchSys implements System {
    checkTime: number = 0;
    dirty: boolean = false;

    // 定义初始化方法（可选）
    setup(_world: World) {
    }

    @read(GlobalState, [Entity, DataC], [Entity, StateC], [Entity, TypeC])
    @write(FrameTime)
    run(read: [Single<GlobalState>, ArrMulti<Entity, DataC>, ArrMulti<Entity, StateC>, ArrMulti<Entity, TypeC>], write: [Single<FrameTime>]) {
        let frame = write[0].data;
        // 一秒检查一次
        if (!this.dirty && frame.curTime() - this.checkTime < 1000) return;
        this.checkTime = frame.curTime();
        let globalState = read[0].data;
        let dataC = read[1];
        let stateC = read[2];
        let typeC = read[3];
        let indexs = globalState.areaIndexs.get(globalState.curArea);
        let [w, h] = globalState.areaMap.get(globalState.curArea);
        let typeList = [];
        // 从左往右 从下往上 取出可以参与匹配的物品
        for (let x = 0; x < w; x++) {
            typeList[x] = [];
            for (let y = 0; y < h; y++) {
                typeList[x][y] = 0;
                let id = indexs[LayerType.Lock][x][y];
                if (id) continue;
                id = indexs[LayerType.Data][x][y];
                if (!id) continue;
                // 有格子正在变化中
                if (stateC.get(id).value !== DataState.Normal) return;
                typeList[x][y] = typeC.get(id).value;
            }
        }
        let lines = findLines(typeList);
        let len = lines.length;
        if (len > 1) {
            let newLines = [];
            for (let i = 0; i < len; i++) {
                if (newLines[i]) continue;
                let list = lines[i].slice();
                for (let j = i + 1; j < len; j++) {
                    for (let [x, y] of lines[j]) {
                        let ind = list.findIndex(([x1, y1]) => x === x1 && y === y1);
                        // 两条线相交 合并为一个数组
                        if (ind > -1) {
                            list.splice(ind, 1);
                            list.push(...lines[j]);
                            newLines[j] = [];
                            break;
                        }
                    }
                }
                newLines[i] = list;
            }
            lines = newLines.filter(v => v.length);
        }
        if (lines.length) {
            for (let list of lines) {
                for (let [x, y] of list) {
                    let id = indexs[LayerType.Data][x][y];
                    stateC.get(id).value = DataState.WaitExplode;
                }
                // TODO 合成炸弹
            }
            return
        }
        let squares = findSquare(typeList);
        for (let v of squares) {
            for (let [i, j] of SquarePosFour) {
                let [x, y] = v.pos;
                let id = indexs[LayerType.Data][x + i][y + j];
                stateC.get(id).value = DataState.WaitExplode;
            }

            // TODO 合成炸弹
        }
    }

    @listenModify(ActionCmd)
    @read(ActionCmd)
    listenActionCmd(e: ModifyEvent, read: [Single<ActionCmd>], write: any) {
        let action = read[0].data;
        // 交换才能合成炸弹
        if (action.cmd === ActionName.change) {
            this.dirty = true;
        }
    }
}

export const findSquare = (matrix: number[][]) => {
    const width = matrix.length;
    const height = matrix[0].length;
    let squares = [];

    // 遍历数组，但不包括最后一行和最后一列
    for (let x = 0; x < width - 1; x++) {
        for (let y = 0; y < height - 1; y++) {
            // 检查2x2正方形的所有值是否相等
            if (matrix[x][y] === matrix[x][y + 1] &&
                matrix[x][y] === matrix[x + 1][y] &&
                matrix[x][y] === matrix[x + 1][y + 1]) {
                squares.push({
                    pos: [x, y],
                    value: matrix[x][y]
                });
            }
        }
    }
    if (squares.length > 0) console.log('findSquare squares ====', squares);

    return squares;
}

export const findLines = (matrix: number[][], minLength: number = 3) => {
    const width = matrix.length;
    const height = matrix[0].length;
    const lines = [];

    // 检查垂直方向
    for (let x = 0; x < width; x++) {
        for (let y = 0; y <= height - minLength; y++) {
            if (!matrix[x][y]) continue;
            let matchCount = 1;
            while (y + matchCount < height && matrix[x][y] === matrix[x][y + matchCount]) {
                matchCount++;
            }
            if (matchCount >= minLength) {
                const line = [];
                for (let i = 0; i < matchCount; i++) {
                    line.push([x, y + i]);
                }
                lines.push(line);
                y += matchCount - 1; // 跳过已经检查过的部分
            }
        }
    }

    // 检查水平方向
    for (let y = 0; y < height; y++) {
        for (let x = 0; x <= width - minLength; x++) {
            if (!matrix[x][y]) continue;
            let matchCount = 1;
            while (x + matchCount < width && matrix[x][y] === matrix[x + matchCount][y]) {
                matchCount++;
            }
            if (matchCount >= minLength) {
                const line = [];
                for (let i = 0; i < matchCount; i++) {
                    line.push([x + i, y]);
                }
                lines.push(line);
                x += matchCount - 1; // 跳过已经检查过的部分
            }
        }
    }
    if (lines.length > 0) console.log('findLines lines ====', lines);

    return lines;
}