import { AwardType, DataType, EventDataType, NormalLockType, SpecialLockType, TypeC } from "../component/type";
import { randomInt } from "pi_math/math";
import { BombType } from "../single/bomb_list";
import { ArrMulti, Entity, World } from "pi_ucore/util/ecs";
import { DataC, DataState, LayerType, StateC } from "../component/data";
import { Rand } from "pi_math/rand";
import { GlobalState } from "../single/global_state";

// 周围4个点
export const AroundPosFour = [[-1, 0], [0, -1], [1, 0], [0, 1]]

// 周围8个点
export const AroundPosEight = [[-1, 0], [-1, -1], [0, -1], [1, -1], [1, 0], [1, 1], [0, 1], [-1, 1]]

// 正方形四个点
export const SquarePosFour = [[0, 0], [0, 1], [1, 1], [1, 0]]

/**
 * 根据权重随机id
 * @param idArr 随机的id列表
 * @param weightArr 随机的权重
 * @returns 随机出的id
 */
export const randomIndexByWeight = <T>(idArr: Array<T>, weightArr: Array<number>, rand?: Rand): T => {
    const t = weightArr.reduce((x, y) => x + y)
    const k = rand ? rand.nextInt(0, t) : randomInt(0, t);
    //处理出一个新的权重区间数组
    const newCompareArr = weightArr.map((x, i) => {
        return weightArr.reduce((v1, v2, k) => {
            if (k > i) return v1
            return v1 + v2
        })
    });
    for (let i = 0; i < newCompareArr.length; i++) {
        if (k <= newCompareArr[i]) return idArr[i]
    }
}

/**
 * 随机位置
 * @param posList 可用坐标
 * @param w 宽
 * @param h 高
 * @param size 范围
 * @returns 
 */
export const randomPos = (posList: [number, number][], rand: Rand, size: [number, number] = [1, 1]) => {
    let ind = rand.nextInt(0, posList.length - 1);
    let pos = posList[ind];
    let list = [];
    for (let i = 0; i < size[0]; i++) {
        for (let j = 0; j < size[1]; j++) {
            let x = pos[0] + i, y = pos[1] + j;
            let ind = posList.findIndex(v => v[0] === x && v[1] === y)
            // 其中一个位置不存在 重新随
            if (ind === -1) return randomPos(posList, rand, size);
            list.push([x, y]);
        }
    }
    for (let [x, y] of list) {
        let ind = posList.findIndex(v => v[0] === x && v[1] === y)
        posList.splice(ind, 1);
    }

    return pos;
}

// 合成炸弹
export const synthesisBomb = (posList: [number, number][], typeList: TypeC[], changeTypeList = true) => {
    if (typeList.length < 4) return {};
    let dataType = 0; // 默认1
    for (let i = 0; i < typeList.length; i++) {
        let v = typeList[i]
        // 非基础物件不能合成
        if (v.value > 8) return {};
        // 不止一种类型 不能合成
        if (dataType && dataType !== v.value) return {};
        dataType = v.value;
    }

    let bombTypes: number[] = [];
    let tempBombTypes: number[] = [];
    let pos: [number, number][] = [];
    let start = 4;
    for (let i = 4; i <= posList.length; i++) {
        const curPosList = posList.slice(i - 4, i);
        let eachBombType = parseEachBoom(curPosList);
        if (!eachBombType) break
        if (tempBombTypes[start - 12]) {//合成三级雷 
            tempBombTypes = [eachBombType];
            start = 3;
            eachBombType = thirdSynthesis(eachBombType)
        } else if (tempBombTypes[start - 8]) {//合成二级雷 
            eachBombType = secondSynthesis(tempBombTypes[start - 8], eachBombType)
            tempBombTypes.push(eachBombType);
        } else {
            tempBombTypes.push(eachBombType);
        }
        bombTypes.push(eachBombType);
        pos.push(posList[i - 4]);
        start++;
    }

    return { bombTypes, pos, dataType, superBombType: null }
}

// 每四个解析雷
const parseEachBoom = (posList: [number, number][]) => {
    if (posList.length <= 3) return 0

    let bombType = 0;
    let xSet = new Map, ySet = new Map;
    for (let [x, y] of posList) {
        let n1 = xSet.get(x) || 0;
        let n2 = ySet.get(y) || 0;
        xSet.set(x, n1 + 1)
        ySet.set(y, n2 + 1);
    }
    if (xSet.size === 4) bombType = BombType.RowBomb;
    else if (ySet.size === 4) bombType = BombType.ColumnBomb;
    else if (xSet.size === 3 && [...ySet.values()][0] === 2) bombType = BombType.AirPlaneBomb;
    else if (ySet.size === 3 && [...xSet.values()][0] === 2) bombType = BombType.AirPlaneBomb;
    else if (xSet.size === 3 || ySet.size === 3) bombType = BombType.SplitBomb;
    else if (xSet.size === 2 && ySet.size === 2) bombType = BombType.NineGridBomb;

    return bombType;
}

// 二次合成
export const secondSynthesis = (bombType1: BombType, bombType2: BombType, isAllBomb: boolean = false) => {
    let ruleList = [
        [BombType.RowBomb, BombType.RowBomb, BombType.RowColumn],
        [BombType.ColumnBomb, BombType.ColumnBomb, BombType.RowColumn],
        [BombType.RowBomb, BombType.ColumnBomb, BombType.RowColumn],
        [BombType.RowBomb, BombType.NineGridBomb, BombType.ThreeRow],
        [BombType.ColumnBomb, BombType.NineGridBomb, BombType.ThreeColumn],
        [BombType.NineGridBomb, BombType.NineGridBomb, BombType.BigGridBomb],
        [BombType.RowBomb, BombType.AirPlaneBomb, BombType.AirplaneRow],
        [BombType.ColumnBomb, BombType.AirPlaneBomb, BombType.AirplaneColumn],
        [BombType.RowBomb, BombType.SplitBomb, BombType.RocketRow],
        [BombType.ColumnBomb, BombType.SplitBomb, BombType.RocketColumn],
        [BombType.NineGridBomb, BombType.AirPlaneBomb, BombType.AirplaneNineGrid],
        [BombType.AirPlaneBomb, BombType.AirPlaneBomb, BombType.ThreeAirplane],
        [BombType.NineGridBomb, BombType.SplitBomb, BombType.RocketNineGrid],
        [BombType.AirPlaneBomb, BombType.SplitBomb, BombType.RocketAirplane],
        [BombType.SplitBomb, BombType.SplitBomb, BombType.BigRocketBomb],
    ]
    for (let [type1, type2, type3] of ruleList) {
        if (bombType1 === type1 && bombType2 === type2) return type3;
        if (bombType2 === type1 && bombType1 === type2) return type3;
    }
}

// 三次合成
export const thirdSynthesis = (bombType: BombType) => {
    let superBombRuleList = [
        [BombType.RowBomb, BombType.SuperColumn],
        [BombType.ColumnBomb, BombType.SuperColumn],
        [BombType.NineGridBomb, BombType.SuperNineGridBomb],
        [BombType.AirPlaneBomb, BombType.SuperAirPlaneBomb],
        [BombType.SplitBomb, BombType.SuperSplitBomb],
    ];
    // 四个全是一样的雷 才会合成超级雷
    for (let [type1, type2] of superBombRuleList) {
        if (bombType === type1) return type2;
    }
}

//合成雷所有形状
const bombShape = [
    [[0, 0], [0, 1], [1, 1], [1, 2]],           //上右z
    [[0, 0], [0, 1], [-1, 1], [-1, 2]],         //上左Z
    [[0, 0], [1, 0], [1, 1], [2, 1]],           //右上Z
    [[0, 0], [1, 0], [1, -1], [2, -1]],         //右下Z
    [[0, 0], [0, -1], [1, -1], [2, -1]],        //下右L
    [[0, 0], [0, -1], [-1, -1], [-2, -1]],      //下左L
    [[0, 0], [0, 1], [1, 1], [2, 1]],           //上右L
    [[0, 0], [0, 1], [-1, 1], [-2, 1]],         //上左L
    [[0, 0], [0, 1], [0, 2], [1, 2]],           //长上右L
    [[0, 0], [0, 1], [0, 2], [-1, 2]],          //长上左L
    [[0, 0], [0, -1], [0, -2], [1, -2]],        //长下右L
    [[0, 0], [0, -1], [0, -2], [-1, -2]],       //长下左L
    [[0, 0], [0, -1], [1, -1], [1, 0]],         //方形
    [[0, 0], [0, 1], [0, 2], [0, 3]],           //竖I
    [[0, 0], [1, 0], [2, 0], [3, 0]],           //横I
];

//自动筛选合成二级雷
export const autoSecondSynthesis = (index: number, waitSynthesis: number[], globalState: GlobalState, dataC: ArrMulti<Entity, DataC>, typeC: ArrMulti<Entity, TypeC>) => {
    const area = globalState.mapCfg.area1;
    let entitys = globalState.areaIndexs.get(globalState.curArea)[LayerType.Data];
    let data = dataC.get(waitSynthesis[index]);
    let centerPos: [number, number] = data.pos;
    let bombs: Map<BombType, [number, number][][]> = new Map();
    let bombTypes: BombType[] = [];
    bombShape.forEach((v, shapeIndex) => {
        let bombType: BombType = shapeIndex < 4 ? BombType.AirPlaneBomb : shapeIndex < 12 ? BombType.SplitBomb : shapeIndex < 13 ? BombType.NineGridBomb : shapeIndex < 14 ? BombType.ColumnBomb : BombType.RowBomb;
        let list = bombs.get(bombType) || [];
        for (let [shapeX, shapeY] of v) {
            let posList: [number, number][] = [];
            for (let [x, y] of v) {
                let realShapePos = [x - shapeX, y - shapeY];
                let realPos: [number, number] = [centerPos[0] + realShapePos[0], centerPos[1] + realShapePos[1]];
                //超出地图 break;
                if (realPos[0] < 0 || realPos[1] < 0 || realPos[0] >= area[0] || realPos[1] >= area[1]) break;
                let entityId = entitys[realPos[0]][realPos[1]];
                if (!entityId) break;
                let type = typeC.get(entityId);
                //不是雷 break;
                if (type.value !== DataType.SynthesisBomb) break;
                //当前雷再待合成雷中且排在index后面 break;
                if (waitSynthesis.findIndex(waitSyn => waitSyn[0] === realPos[0] && waitSyn[1] === realPos[1]) > index) break;
                posList.push(realPos);
            }
            if (posList.length === 4) list.push(posList);
        }
        if (list.length) {
            bombs.set(bombType, list);
            if (!bombTypes.includes(bombType)) bombTypes.push(bombType);
        }
    })
    // 随机雷
    if (!bombTypes.length) return {};
    // 随机雷类型
    let randomBombType = bombTypes[Math.floor(Math.random() * bombTypes.length)];
    // 随机雷形状位置
    let list = bombs.get(randomBombType);
    let randomPos: [number, number][] = list[Math.floor(Math.random() * list.length)];

    //生成的二级雷
    let sceondBombType: BombType = secondSynthesis(typeC.get(waitSynthesis[index]).subType as BombType, randomBombType);

    return { centerPos: centerPos, bombType: sceondBombType, bombShapePos: randomPos };
}

// 获取类型列表
export const getDataTypeList = (world: World, dataType: DataType, subType?: NormalLockType | SpecialLockType | BombType | AwardType | EventDataType) => {
    const typeC = world.fetchComponent(Entity, TypeC) as ArrMulti<Entity, TypeC>;
    const stateC = world.fetchComponent(Entity, StateC) as ArrMulti<Entity, StateC>;
    let list = [];
    for (let type of typeC.iter()) {
        if (!type) continue;
        let id = (<any>type).__id__;
        // 排除 即将被销毁
        if (type.needBombTimes === 0 && stateC.get(id)?.value === DataState.Exploded) continue;
        if (type.value === dataType && (!subType || type.subType === subType)) list.push(id);
    }
    return list;
}

// 是否有水域
export const isHaveWater = (indexs: number[][][], [x, y]: number[], w: number, h: number) => {
    // 自身如果是个草垛则会阻挡扩展
    if (getGrid(indexs, LayerType.Lawn, x, y) > 0) return null;
    for (let [i, j] of AroundPosFour) {
        let x1 = x + i, y1 = y + j;
        if (x1 < 0 || x1 >= w || y1 < 0 || y1 >= h) continue;
        let lawnId = getGrid(indexs, LayerType.Lawn, x1, y1);
        if (lawnId === -1) {
            return [x1, y1];
        }
    }

    return null;
}

// 是否周围全是草垛或障碍物
export const isAllHaystack = (indexs: number[][][], [x, y]: number[], w: number, h: number) => {
    let fg = true;
    for (let [i, j] of AroundPosFour) {
        let x1 = x + i, y1 = y + j;
        if (x1 < 0 || x1 >= w || y1 < 0 || y1 >= h) continue;
        let lawnId = getGrid(indexs, LayerType.Lawn, x1, y1);
        if (lawnId <= 0 && !getGrid(indexs, LayerType.Lock, x1, y1)) {
            fg = false;
        }
    }

    return fg;
}

export type Coordinate = [number, number, number]; // [x,y,entity]

// 将坐标按是否相邻分堆
export const groupAdjacentCoordinates = (coords: Coordinate[]) => {
    const parent: number[] = []; // 并查集的父节点数组
    const rank: number[] = [];   // 并查集的秩（用于优化合并）

    // 初始化并查集
    for (let i = 0; i < coords.length; i++) {
        parent[i] = i; // 每个坐标初始时是自己的父节点
        rank[i] = 1;    // 初始秩为 1
    }

    // 查找根节点（带路径压缩）
    function find(u: number): number {
        if (parent[u] !== u) {
            parent[u] = find(parent[u]); // 路径压缩
        }
        return parent[u];
    }

    // 合并两个集合（按秩合并）
    function union(u: number, v: number) {
        const rootU = find(u);
        const rootV = find(v);
        if (rootU !== rootV) {
            if (rank[rootU] > rank[rootV]) {
                parent[rootV] = rootU;
            } else if (rank[rootU] < rank[rootV]) {
                parent[rootU] = rootV;
            } else {
                parent[rootV] = rootU;
                rank[rootU]++;
            }
        }
    }

    // 遍历所有坐标对，合并相邻的坐标
    for (let i = 0; i < coords.length; i++) {
        for (let j = i + 1; j < coords.length; j++) {
            if (isAdjacent(coords[i], coords[j])) {
                union(i, j); // 合并相邻的坐标
            }
        }
    }

    // 根据并查集的分组结果，构建最终的堆
    const groupMap: Map<number, Coordinate[]> = new Map();
    for (let i = 0; i < coords.length; i++) {
        const root = find(i);
        if (!groupMap.has(root)) {
            groupMap.set(root, []);
        }
        groupMap.get(root)!.push(coords[i]);
    }

    // 返回分组结果
    return Array.from(groupMap.values());
}

// 判断两个坐标是否相邻
const isAdjacent = (coord1: Coordinate, coord2: Coordinate) => {
    const [x1, y1] = coord1;
    const [x2, y2] = coord2;
    return Math.abs(x1 - x2) <= 1 && Math.abs(y1 - y2) <= 1;
}

// 按距离分组
export const groupCoordinates = (coordinates: Coordinate[], center: Coordinate) => {
    // 计算距离
    const calculateDistance = (point: Coordinate): number => {
        return Math.max(Math.abs(point[0] - center[0]), Math.abs(point[1] - center[1]));
    };

    // 按距离分组
    const groups: { [key: number]: Coordinate[] } = {};
    coordinates.forEach(coord => {
        let distance = calculateDistance(coord);
        // 将中心坐标视为距离为1
        if (distance === 0) distance = 1;
        if (!groups[distance]) {
            groups[distance] = [];
        }
        groups[distance].push(coord);
    });

    // 将分组结果转换为数组并按距离排序
    const sortedDistances = Object.keys(groups).map(Number).sort((a, b) => a - b);
    const result: Coordinate[][] = sortedDistances.map(distance => groups[distance]);

    return result;
}

// 获取格子
export const getGrid = (indexs: number[][][], layer: LayerType, x: number, y: number) => {
    if (x < 0 || y < 0 || x >= indexs[layer].length || y >= indexs[layer][x].length) return 0;
    return indexs[layer][x][y];
}

// 设置格子
export const setGrid = (indexs: number[][][], layer: LayerType, x: number, y: number, value: number) => {
    if (x < 0 || y < 0 || x >= indexs[layer].length || y >= indexs[layer][x].length) return;
    indexs[layer][x][y] = value;
}


// 三消world
let ThreeMatchWorld = null;
export const setThreeMathWorld = (world: World) => {
    ThreeMatchWorld = world;
}
export const getThreeMathWorld = () => {
    return ThreeMatchWorld;
}
