import { Entity, World } from "pi_ucore/util/ecs"
import { FrameTime } from "pi_scene/scene/single/global";
import { ActionCmd, ActionName } from "../single/action_cmd";
import { GameState, GlobalState } from "../single/global_state";
import { getThreeMathWorld } from "./base";
import { DataTypeList } from "../single/data_type_list";
import { AnimList } from "../single/anim_list";

// 点击
export const click = (world: World, pos: [number, number]) => {
    const actionCmd = world.fetchSingle(ActionCmd).data;
    actionCmd.addCmd(ActionName.click, pos, '');
}

// 使用炸弹
export const useBomb = (world: World, posList: [number, number][]) => {
    const actionCmd = world.fetchSingle(ActionCmd).data;
    actionCmd.addCmd(ActionName.useBomb, posList[0], JSON.stringify(posList));
}

// 交换
export const swap = (world: World, posList: [number, number][]) => {
    const actionCmd = world.fetchSingle(ActionCmd).data;
    actionCmd.addCmd(ActionName.swap, posList[0], JSON.stringify(posList));
}

// 随机混乱
export const shuffle = (world: World, posList: [number, number][]) => {
    const actionCmd = world.fetchSingle(ActionCmd).data;
    actionCmd.addCmd(ActionName.shuffle, posList[0], JSON.stringify(posList));
}

// 添加合成雷
export const addSynthesisBomb = (world: World, count: number, fixBooms: number[], startPos: [number, number]) => {
    const actionCmd = world.fetchSingle(ActionCmd).data;
    actionCmd.addCmd(ActionName.addSynthesisBomb, [0, 0], JSON.stringify({ count, fixBooms, startPos }));
}

// 跳过游戏 测试性能
export const skipGame = (testList: any[]) => {
    let world = getThreeMathWorld();
    const arr = [""].map(s => world.getDispatcher(s));
    const frame = world.fetchSingle(FrameTime).data;
    const globalState: GlobalState = world.fetchSingle(GlobalState).data;
    const action = world.fetchSingle(ActionCmd).data;
    let start = [frame.count, Date.now()]
    let c = testList.shift();
    action.addCmd(c.cmd, c.pos, c.arg);
    console.warn('游戏开始', frame.count, Date.now())
    while (testList.length > 0) {
        frame.count += 1;
        for (let d of arr) {
            d.run();
        }
        if (globalState.canOperate()) {
            let c = testList.shift();
            action.addCmd(c.cmd, c.pos, c.arg);
        }
    }
    let time = Date.now() - start[1], count = frame.count - start[0];
    console.warn('游戏结束', frame.count, Date.now(), '消耗帧：', count, '消耗时间：', time, '平均耗时：', time / count)
}

// 恢复游戏状态
export const recoverGame = (testList: any[]) => {
    let world = getThreeMathWorld();
    const arr = [""].map(s => world.getDispatcher(s));
    const frame = world.fetchSingle(FrameTime).data;
    const globalState = world.fetchSingle(GlobalState).data;
    const action = world.fetchSingle(ActionCmd).data;
    const animList = world.fetchSingle(AnimList).data;

    let c = testList.shift();
    action.addCmd(c.cmd, c.pos, c.arg);
    startRun(frame, globalState, action, animList, arr, testList);
}

const startRun = (frame: FrameTime, globalState: GlobalState, action: ActionCmd, animList: AnimList, arr: any[], testList: any[]) => {
    frame.count += 1;
    for (let d of arr) {
        d.run();
    }
    if (globalState.canOperate() && animList.animList.size == 0) {
        let c = testList.shift();
        action.addCmd(c.cmd, c.pos, c.arg);
    }
    if (testList.length > 0) {
        setTimeout(() => {
            startRun(frame, globalState, action, animList, arr, testList);
        }, 0);
    }
}

// 获取物品类型数组
const getAllDataTypeList = () => {
    let world = getThreeMathWorld();
    const dataTypeList = world.fetchSingle(DataTypeList).data;
    return dataTypeList.list;
}

(<any>window).skipGame = skipGame;
(<any>window).recoverGame = recoverGame;
(<any>window).getAllDataTypeList = getAllDataTypeList;